@@ -1,5 +1,4 @@
 import six
-import warnings
 
 from django.conf import settings
 from django.contrib import messages
@@ -8,11 +7,11 @@ from django.contrib.auth.views import redirect_to_login
 from django.core import serializers
 from django.core.exceptions import ImproperlyConfigured, PermissionDenied
 from django.core.serializers.json import DjangoJSONEncoder
-from django.core.urlresolvers import reverse
-from django.http import HttpResponse
+from django.core.urlresolvers import resolve, reverse
+from django.http import HttpResponse, HttpResponseBadRequest
+from django.shortcuts import redirect
 from django.utils.decorators import method_decorator
 from django.utils.encoding import force_text
-from django.views.generic import CreateView
 from django.views.decorators.csrf import csrf_exempt
 
 ## Django 1.5+ compat
@@ -22,40 +21,12 @@ except ImportError:  # pragma: no cover
     from django.utils import simplejson as json
 
 
-class CreateAndRedirectToEditView(CreateView):
-    """
-    Subclass of CreateView which redirects to the edit view.
-    Requires property `success_url_name` to be set to a
-    reversible url that uses the objects pk.
-    """
-    success_url_name = None
-
-    def dispatch(self, request, *args, **kwargs):
-        warnings.warn(
-            "CreateAndRedirectToEditView is deprecated and will be "
-            "removed in a future release.", PendingDeprecationWarning)
-        return super(CreateAndRedirectToEditView, self).dispatch(
-            request, *args, **kwargs)
-
-    def get_success_url(self):
-        # First we check for a name to be provided on the view object.
-        # If one is, we reverse it and finish running the method,
-        # otherwise we raise a configuration error.
-        if self.success_url_name:
-            self.success_url = reverse(
-                self.success_url_name, kwargs={'pk': self.object.pk})
-            return super(CreateAndRedirectToEditView, self).get_success_url()
-
-        raise ImproperlyConfigured(
-            "No URL to reverse. Provide a success_url_name.")
-
-
 class AccessMixin(object):
     """
     'Abstract' mixin that gives access mixins the same customizable
     functionality.
     """
-    login_url = settings.LOGIN_URL  # LOGIN_URL from project settings
+    login_url = None
     raise_exception = False  # Default whether to raise an exception to none
     redirect_field_name = REDIRECT_FIELD_NAME  # Set by django.contrib.auth
 
@@ -63,13 +34,13 @@ class AccessMixin(object):
         """
         Override this method to customize the login_url.
         """
-        if self.login_url is None:
+        login_url = self.login_url or settings.LOGIN_URL
+        if not login_url:
             raise ImproperlyConfigured(
-                "%(cls)s is missing the login_url. "
-                "Define %(cls)s.login_url or override "
+                "Define %(cls)s.login_url or settings.LOGIN_URL or override "
                 "%(cls)s.get_login_url()." % {"cls": self.__class__.__name__})
 
-        return force_text(self.login_url)
+        return force_text(login_url)
 
     def get_redirect_field_name(self):
         """
@@ -285,7 +256,8 @@ class GroupRequiredMixin(AccessMixin):
     def get_group_required(self):
         if self.group_required is None or (
                 not isinstance(self.group_required,
-                               (list, tuple) + six.string_types)):
+                               (list, tuple) + six.string_types)
+        ):
 
             raise ImproperlyConfigured(
                 "'GroupRequiredMixin' requires "
@@ -295,13 +267,13 @@ class GroupRequiredMixin(AccessMixin):
 
     def check_membership(self, group):
         """ Check required group(s) """
-        if not group in self.request.user.groups.values_list('name',
-                                                             flat=True):
-            return False
-        return True
+        return group in self.request.user.groups.values_list("name", flat=True)
 
     def dispatch(self, request, *args, **kwargs):
-        in_group = self.check_membership(self.get_group_required())
+        self.request = request
+        in_group = False
+        if self.request.user.is_authenticated():
+            in_group = self.check_membership(self.get_group_required())
 
         if not in_group:
             if self.raise_exception:
@@ -315,6 +287,42 @@ class GroupRequiredMixin(AccessMixin):
             request, *args, **kwargs)
 
 
+class UserPassesTestMixin(AccessMixin):
+    """
+    CBV Mixin allows you to define test that every user should pass
+    to get access into view.
+
+    Class Settings
+        `test_func` - This is required to be a method that takes user
+            instance and return True or False after checking conditions.
+        `login_url` - the login url of site
+        `redirect_field_name` - defaults to "next"
+        `raise_exception` - defaults to False - raise 403 if set to True
+    """
+
+    def test_func(self, user):
+        raise NotImplementedError(
+                "%(cls)s is missing implementation of the "
+                "test_func method. You should write one." % {
+                        "cls": self.__class__.__name__})
+
+    def get_test_func(self):
+        return getattr(self, "test_func")
+
+    def dispatch(self, request, *args, **kwargs):
+        user_test_result = self.get_test_func()(request.user)
+
+        if not user_test_result:  # If user don't pass the test
+            if self.raise_exception:  # *and* if an exception was desired
+                raise PermissionDenied
+            else:
+                return redirect_to_login(request.get_full_path(),
+                                         self.get_login_url(),
+                                         self.get_redirect_field_name())
+        return super(UserPassesTestMixin, self).dispatch(
+            request, *args, **kwargs)
+
+
 class UserFormKwargsMixin(object):
     """
     CBV mixin which puts the user from the request into the form kwargs.
@@ -471,23 +479,23 @@ class JSONResponseMixin(object):
     A mixin that allows you to easily serialize simple data such as a dict or
     Django models.
     """
-    content_type = "application/json"
+    content_type = u"application/json"
     json_dumps_kwargs = None
 
     def get_content_type(self):
         if self.content_type is None:
             raise ImproperlyConfigured(
-                "%(cls)s is missing a content type. "
-                "Define %(cls)s.content_type, or override "
-                "%(cls)s.get_content_type()." % {
-                "cls": self.__class__.__name__}
+                u"%(cls)s is missing a content type. "
+                u"Define %(cls)s.content_type, or override "
+                u"%(cls)s.get_content_type()." % {
+                u"cls": self.__class__.__name__}
             )
         return self.content_type
 
     def get_json_dumps_kwargs(self):
         if self.json_dumps_kwargs is None:
             self.json_dumps_kwargs = {}
-        self.json_dumps_kwargs.setdefault('ensure_ascii', False)
+        self.json_dumps_kwargs.setdefault(u'ensure_ascii', False)
         return self.json_dumps_kwargs
 
     def render_json_response(self, context_dict, status=200):
@@ -496,7 +504,8 @@ class JSONResponseMixin(object):
         or other complex or custom objects.
         """
         json_context = json.dumps(context_dict, cls=DjangoJSONEncoder,
-                                  **self.get_json_dumps_kwargs())
+                                  **self.get_json_dumps_kwargs()).encode(
+                                  u'utf-8')
         return HttpResponse(json_context,
                             content_type=self.get_content_type(),
                             status=status)
@@ -506,7 +515,7 @@ class JSONResponseMixin(object):
         Serializes objects using Django's builtin JSON serializer. Additional
         kwargs can be used the same way for django.core.serializers.serialize.
         """
-        json_data = serializers.serialize("json", objects, **kwargs)
+        json_data = serializers.serialize(u"json", objects, **kwargs)
         return HttpResponse(json_data, content_type=self.get_content_type())
 
 
@@ -520,7 +529,7 @@ class AjaxResponseMixin(object):
         request_method = request.method.lower()
 
         if request.is_ajax() and request_method in self.http_method_names:
-            handler = getattr(self, '%s_ajax' % request_method,
+            handler = getattr(self, u"{0}_ajax".format(request_method),
                               self.http_method_not_allowed)
             self.request = request
             self.args = args
@@ -543,6 +552,57 @@ class AjaxResponseMixin(object):
         return self.get(request, *args, **kwargs)
 
 
+class JsonRequestResponseMixin(JSONResponseMixin):
+    """
+    Extends JSONResponseMixin.  Attempts to parse request as JSON.  If request
+    is properly formatted, the json is saved to self.request_json as a Python
+    object.  request_json will be None for imparsible requests.
+    Set the attribute require_json to True to return a 400 "Bad Request" error
+    for requests that don't contain JSON.
+
+    Note: To allow public access to your view, you'll need to use the
+    csrf_exempt decorator or CsrfExemptMixin.
+
+    Example Usage:
+
+        class SomeView(CsrfExemptMixin, JsonRequestResponseMixin):
+            def post(self, request, *args, **kwargs):
+                do_stuff_with_contents_of_request_json()
+                return self.render_json_response(
+                    {'message': 'Thanks!'})
+    """
+    require_json = False
+    error_response_dict = {u"errors": [u"Improperly formatted request"]}
+
+    def render_bad_request_response(self, error_dict=None):
+        if error_dict is None:
+            error_dict = self.error_response_dict
+        json_context = json.dumps(
+            error_dict,
+            cls=DjangoJSONEncoder,
+            **self.get_json_dumps_kwargs()
+        ).encode(u'utf-8')
+        return HttpResponseBadRequest(
+            json_context, content_type=self.get_content_type())
+
+    def get_request_json(self):
+        try:
+            return json.loads(self.request.body.decode(u'utf-8'))
+        except ValueError:
+            return None
+
+    def dispatch(self, request, *args, **kwargs):
+        self.request = request
+        self.args = args
+        self.kwargs = kwargs
+
+        self.request_json = self.get_request_json()
+        if self.require_json and self.request_json is None:
+            return self.render_bad_request_response()
+        return super(JsonRequestResponseMixin, self).dispatch(
+           request, *args, **kwargs)
+
+
 class OrderableListMixin(object):
     """
     Mixin allows your users to order records using GET parameters
@@ -607,6 +667,55 @@ class OrderableListMixin(object):
         return self.get_ordered_queryset(unordered_queryset)
 
 
+class CanonicalSlugDetailMixin(object):
+    """
+    A mixin that enforces a canonical slug in the url.
+
+    If a urlpattern takes a object's pk and slug as arguments and the slug url
+    argument does not equal the object's canonical slug, this mixin will
+    redirect to the url containing the canonical slug.
+    """
+    def dispatch(self, request, *args, **kwargs):
+        # Set up since we need to super() later instead of earlier.
+        self.request = request
+        self.args = args
+        self.kwargs = kwargs
+
+        # Get the current object, url slug, and
+        # urlpattern name (namespace aware).
+        obj = self.get_object()
+        slug = self.kwargs.get(self.slug_url_kwarg, None)
+        match = resolve(request.path_info)
+        url_parts = match.namespaces
+        url_parts.append(match.url_name)
+        current_urlpattern = ":".join(url_parts)
+
+        # Figure out what the slug is supposed to be.
+        if hasattr(obj, "get_canonical_slug"):
+            canonical_slug = obj.get_canonical_slug()
+        else:
+            canonical_slug = self.get_canonical_slug()
+
+        # If there's a discrepancy between the slug in the url and the
+        # canonical slug, redirect to the canonical slug.
+        if canonical_slug != slug:
+            return redirect(current_urlpattern, pk=obj.pk, slug=canonical_slug,
+                            permanent=True)
+
+        return super(CanonicalSlugDetailMixin, self).dispatch(
+            request, *args, **kwargs)
+
+    def get_canonical_slug(self):
+        """
+        Override this method to customize what slug should be considered
+        canonical.
+
+        Alternatively, define the get_canonical_slug method on this view's
+        object class. In that case, this method will never be called.
+        """
+        return self.get_object().slug
+
+
 class FormValidMessageMixin(object):
     """
     Mixin allows you to set static message which is displayed by
@@ -667,7 +776,8 @@ class FormInvalidMessageMixin(object):
                     self.__class__.__name__)
             )
 
-        if not isinstance(self.form_invalid_message, six.string_types):
+        if not isinstance(self.form_invalid_message,
+                          (six.string_types, six.text_type)):
             raise ImproperlyConfigured(
                 '{0}.form_invalid_message must be a str or unicode '
                 'object.'.format(self.__class__.__name__)
